@@ -1,20 +1,20 @@
 /*
    +----------------------------------------------------------------------+
-   | PHP Version 5 / Imagick	                                          |
+   | PHP Version 5 / Imagick											  |
    +----------------------------------------------------------------------+
-   | Copyright (c) 2006-2008 Mikko Koppanen, Scott MacVicar               |
-   | Imagemagick (c) ImageMagick Studio LLC                               |
+   | Copyright (c) 2006-2009 Mikko Koppanen, Scott MacVicar				  |
+   | Imagemagick (c) ImageMagick Studio LLC								  |
    +----------------------------------------------------------------------+
-   | This source file is subject to version 3.01 of the PHP license,      |
-   | that is bundled with this package in the file LICENSE, and is        |
-   | available through the world-wide-web at the following url:           |
-   | http://www.php.net/license/3_01.txt                                  |
-   | If you did not receive a copy of the PHP license and are unable to   |
-   | obtain it through the world-wide-web, please send a note to          |
-   | license@php.net so we can mail you a copy immediately.               |
+   | This source file is subject to version 3.01 of the PHP license,	  |
+   | that is bundled with this package in the file LICENSE, and is		  |
+   | available through the world-wide-web at the following url:			  |
+   | http://www.php.net/license/3_01.txt								  |
+   | If you did not receive a copy of the PHP license and are unable to	  |
+   | obtain it through the world-wide-web, please send a note to		  |
+   | license@php.net so we can mail you a copy immediately.				  |
    +----------------------------------------------------------------------+
-   | Author: Mikko Kopppanen <mkoppanen@php.net>                          |
-   |         Scott MacVicar <scottmac@php.net>                            |
+   | Author: Mikko Kopppanen <mkoppanen@php.net>						  |
+   |		 Scott MacVicar <scottmac@php.net>							  |
    +----------------------------------------------------------------------+
 */
 
@@ -46,6 +46,111 @@ MagickBooleanType php_imagick_progress_monitor(const char *text, const MagickOff
 	return MagickTrue;	
 }
 
+/*
+	type 1 = MagickWriteImageFile
+	type 2 = MagickWriteImagesFile
+	
+	type 3 = MagickReadImageFile
+	type 4 = MagickPingImageFile
+
+	return values:
+		0 - no error
+		1 - error and exception has been thrown
+		2 - error but exception needs to be thrown
+*/
+int php_imagick_stream_handler(php_imagick_object *intern, php_stream *stream, char *filename, int type TSRMLS_DC)
+{
+#if ZEND_MODULE_API_NO > 20060613 
+	zend_error_handling error_handling;
+#endif
+	FILE *fp;
+	MagickBooleanType status = MagickFalse;
+
+#if ZEND_MODULE_API_NO > 20060613 
+	zend_replace_error_handling(EH_THROW, php_imagick_exception_class_entry, &error_handling TSRMLS_CC);
+#else
+	php_set_error_handling(EH_THROW, php_imagick_exception_class_entry TSRMLS_CC);
+#endif	
+
+	if (php_stream_can_cast(stream, PHP_STREAM_AS_STDIO | PHP_STREAM_CAST_INTERNAL) == FAILURE) {
+		goto return_on_error;
+	}
+
+	if (php_stream_cast(stream, PHP_STREAM_AS_STDIO | PHP_STREAM_CAST_INTERNAL, (void*)&fp, 0) == FAILURE) {
+		goto return_on_error;
+	}
+	
+#if ZEND_MODULE_API_NO > 20060613 
+	zend_restore_error_handling(&error_handling TSRMLS_CC);
+#else
+	php_set_error_handling(EH_NORMAL, NULL TSRMLS_CC);
+#endif	
+
+	/* php_stream_cast returns warning on some streams but still does not return FAILURE */
+	if (EG(exception)) {
+		return 1;
+	} else {
+		return 2;
+	}
+
+	if (type == IMAGICK_WRITE_IMAGE_FILE) {
+		status = MagickWriteImageFile(intern->magick_wand, fp);
+	} else if (type == IMAGICK_WRITE_IMAGES_FILE) {
+		status = MagickWriteImagesFile(intern->magick_wand, fp);
+	} else if (type == IMAGICK_READ_IMAGE_FILE) {		
+		status = MagickReadImageFile(intern->magick_wand, fp);
+	} else if (type == IMAGICK_PING_IMAGE_FILE) {
+		status = MagickPingImageFile(intern->magick_wand, fp);
+	} 
+
+	if (status == MagickFalse) {
+		return 2;
+	}
+	
+	if (filename) {
+		MagickSetImageFilename(intern->magick_wand, filename);
+		IMAGICK_CORRECT_ITERATOR_POSITION(intern);
+	}	
+	
+	return 0;
+	
+return_on_error:
+#if ZEND_MODULE_API_NO > 20060613 
+	zend_restore_error_handling(&error_handling TSRMLS_CC);
+#else
+	php_set_error_handling(EH_NORMAL, NULL TSRMLS_CC);
+#endif
+	if (EG(exception)) {
+		return 1;
+	} else {
+		return 2;
+	}
+}
+
+zend_bool php_imagick_validate_map(const char *map TSRMLS_DC)
+{
+	zend_bool match;
+	char *p = map;
+	char allow_map[] = { 'R', 'G', 'B', 
+						 'A', 'O', 'C', 
+						 'Y', 'M', 'K', 
+						 'I', 'P' };
+
+	while (*p != '\0') {
+		char *it = allow_map;
+		match = 0;
+		while(*it != '\0') {
+			if (*(it++) == *p) {
+				match = 1;
+			}
+		}
+		if (!match) {
+			return 0;
+		}
+		*(p++);
+	}
+	return 1;
+}
 
 int count_occurences_of(char needle, char *haystack TSRMLS_DC)
 {
@@ -56,7 +161,6 @@ int count_occurences_of(char needle, char *haystack TSRMLS_DC)
 	}
 
 	while (*haystack != '\0') {
-
 		if (*(haystack++) == needle) {
 			occurances++;
 		}
@@ -148,6 +252,85 @@ double *get_double_array_from_zval(zval *param_array, long *num_elements TSRMLS_
 	return double_array;
 }
 
+long *get_long_array_from_zval(zval *param_array, long *num_elements TSRMLS_DC)
+{
+	zval **ppzval;
+	HashTable *ht;
+	long *long_array = NULL;
+	long elements, i;
+
+	*num_elements = 0;
+	elements = zend_hash_num_elements(Z_ARRVAL_P(param_array));
+
+	if (elements == 0) {
+		return long_array;
+	}
+
+	long_array = emalloc(sizeof(long) * elements);
+	ht = Z_ARRVAL_P(param_array);
+
+	zend_hash_internal_pointer_reset(ht);
+
+	for (i = 0 ; i < elements ; i++) {
+		if (zend_hash_get_current_data(ht, (void**)&ppzval) == FAILURE) {
+			efree(long_array);
+			long_array = NULL;
+			return long_array;
+		}
+		
+		if ((Z_TYPE_PP(ppzval) == IS_LONG) || (Z_TYPE_PP(ppzval) == IS_DOUBLE)) {
+			long_array[i] = Z_LVAL_PP(ppzval);
+		} else {
+			efree(long_array);
+			long_array = NULL;
+			return long_array;
+		}
+		zend_hash_move_forward(ht);
+	}
+	*num_elements = elements;
+	return long_array;
+}
+
+unsigned char *get_char_array_from_zval(zval *param_array, long *num_elements TSRMLS_DC)
+{
+	zval **ppzval;
+	HashTable *ht;
+	unsigned char *char_array = NULL;
+	long elements, i;
+
+	*num_elements = 0;
+	elements = zend_hash_num_elements(Z_ARRVAL_P(param_array));
+
+	if (elements == 0) {
+		return char_array;
+	}
+
+	char_array = emalloc(sizeof(char) * elements);
+	ht = Z_ARRVAL_P(param_array);
+
+	zend_hash_internal_pointer_reset(ht);
+
+	for (i = 0 ; i < elements ; i++) {
+
+		if (zend_hash_get_current_data(ht, (void**)&ppzval) == FAILURE) {
+			efree(char_array);
+			char_array = NULL;
+			return char_array;
+		}
+		
+		if ((Z_TYPE_PP(ppzval) == IS_LONG) || (Z_TYPE_PP(ppzval) == IS_DOUBLE)) {
+			char_array[i] = (unsigned char)Z_LVAL_PP(ppzval);
+		} else {
+			efree(char_array);
+			char_array = NULL;
+			return char_array;
+		}
+		zend_hash_move_forward(ht);
+	}
+	*num_elements = elements;
+	return char_array;
+}
+
 int check_configured_font(char *font, int font_len TSRMLS_DC)
 {
 	int retval = 0;
@@ -159,7 +342,7 @@ int check_configured_font(char *font, int font_len TSRMLS_DC)
 
 	for(i = 0 ; i < num_fonts ; i++) {
 		/* Let's see if the font is among configured fonts */
-		if (strncmp(fonts[i], font, font_len) == 0) {
+		if (strncasecmp(fonts[i], font, font_len) == 0) {
 			retval = 1;
 			break;
 		}
@@ -169,50 +352,6 @@ int check_configured_font(char *font, int font_len TSRMLS_DC)
 	return retval;
 }
 
-int read_image_into_magickwand(php_imagick_object *intern, char *filename, int type TSRMLS_DC)
-{
-	int error = IMAGICK_READ_WRITE_NO_ERROR;
-	MagickBooleanType status;
-	char *absolute;
-
-	if (!filename) {
-		return IMAGICK_READ_WRITE_NO_ERROR;
-	}
-
-	absolute = expand_filepath(filename, NULL TSRMLS_CC);
-
-	if (!absolute) {
-		return IMAGICK_READ_WRITE_NO_ERROR;
-	}
-
-	if (strlen(absolute) > MAXPATHLEN) {
-		efree(absolute);
-		return IMAGICK_READ_WRITE_FILENAME_TOO_LONG;
-	}
-
-	IMAGICK_SAFE_MODE_CHECK(absolute, error);
-
-	if (error != IMAGICK_READ_WRITE_NO_ERROR) {
-		efree(absolute);
-		return error;
-	}
-
-	if (type == 1) {
-		status = MagickReadImage(intern->magick_wand, absolute);
-	} else {
-		status = MagickPingImage(intern->magick_wand, absolute);
-	}
-
-	efree(absolute);
-
-	if (status == MagickFalse) {
-		return IMAGICK_READ_WRITE_UNDERLYING_LIBRARY;
-	}
-	
-	IMAGICK_CORRECT_ITERATOR_POSITION(intern);
-	return IMAGICK_READ_WRITE_NO_ERROR;
-}
-
 int check_write_access(char *absolute TSRMLS_DC)
 {
 	/* Check if file exists */
@@ -279,43 +418,44 @@ zend_bool crop_thumbnail_image(MagickWand *magick_wand, long desired_width, long
 		}
 		return 1;
 	}
-	
-	if (((double)orig_width / (double)desired_width) > ((double)orig_height / (double)desired_height)) {
-		ratio = (double)orig_height / (double)desired_height;
-		image_width = (double)orig_width / (double)ratio;
+
+	if (((double)orig_width - (double)desired_width) > ((double)orig_height - (double)desired_height)) {
+		ratio		 = (double)orig_height / (double)desired_height;
+		image_width	 = (double)orig_width / (double)ratio;
 		image_height = desired_height;
-		
-		crop_x = ((double)image_width - (double)desired_width) / 2;
+		crop_x		 = ((double)image_width - (double)desired_width) / 2;
 	} else {
-		ratio = (double)orig_width / (double)desired_width;
+		ratio		 = (double)orig_width / (double)desired_width;
 		image_height = (double)orig_height / (double)ratio;
-		image_width = desired_width;
-		
-		crop_y = ((double)image_height - (double)desired_height) / 2;
+		image_width	 = desired_width;
+		crop_y		 = ((double)image_height - (double)desired_height) / 2;
+	}
+	
+	if (desired_width != orig_width && desired_height != orig_height) {
+		if (!MagickThumbnailImage(magick_wand, image_width, image_height)) {
+			return 0;
+		}
 	}
 
-	if (!MagickThumbnailImage(magick_wand, image_width, image_height)) {
-		return 0;
-	}
-	
 	if (!MagickCropImage(magick_wand, desired_width, desired_height, crop_x, crop_y)) {
 		return 0;
 	}
 	
+	MagickSetImagePage(magick_wand, desired_width, desired_height, 0, 0);
 	return 1;
 }
 
 void deallocate_wands(MagickWand *magick, DrawingWand *draw, PixelWand *pixel TSRMLS_DC)
 {
-	if (magick != (MagickWand *)NULL && IsMagickWand(magick)) {
+	if (magick != (MagickWand *)NULL) {
 		magick = (MagickWand *)DestroyMagickWand(magick);
 	}
 
-	if (draw != (DrawingWand *)NULL && IsDrawingWand(draw)) {
+	if (draw != (DrawingWand *)NULL) {
 		draw = (DrawingWand *)DestroyDrawingWand(draw);
 	}
 
-	if (pixel != (PixelWand *)NULL && IsPixelWand(pixel)) {
+	if (pixel != (PixelWand *)NULL) {
 		pixel = (PixelWand *)DestroyPixelWand(pixel);
 	}
 }
@@ -459,7 +599,7 @@ char *get_pseudo_filename(char *pseudo_string TSRMLS_DC)
 int write_image_from_filename(php_imagick_object *intern, char *filename, zend_bool adjoin, int type TSRMLS_DC)
 {
 	int pos = 0, occurances = 0, add_format = 0, error = IMAGICK_READ_WRITE_NO_ERROR;
-	char *buffer, *filepath, *final_filename, c;
+	char *buffer, *filepath, c;
 	MagickBooleanType status;
 
 	occurances = count_occurences_of(':', filename TSRMLS_CC);
@@ -543,6 +683,7 @@ int write_image_from_filename(php_imagick_object *intern, char *filename, zend_b
 	return IMAGICK_READ_WRITE_NO_ERROR;
 }
 
+
 void initialize_imagick_constants()
 {
 	TSRMLS_FETCH();
@@ -621,39 +762,39 @@ void initialize_imagick_constants()
 	IMAGICK_REGISTER_CONST_LONG("MONTAGEMODE_FRAME", FrameMode);
 	IMAGICK_REGISTER_CONST_LONG("MONTAGEMODE_UNFRAME", UnframeMode);
 	IMAGICK_REGISTER_CONST_LONG("MONTAGEMODE_CONCATENATE", ConcatenateMode);
-	IMAGICK_REGISTER_CONST_LONG("STYLE_NORMAL", NormalStyle );
-	IMAGICK_REGISTER_CONST_LONG("STYLE_ITALIC", ItalicStyle );
-	IMAGICK_REGISTER_CONST_LONG("STYLE_OBLIQUE", ObliqueStyle );
-	IMAGICK_REGISTER_CONST_LONG("STYLE_ANY", AnyStyle );
-	IMAGICK_REGISTER_CONST_LONG("FILTER_UNDEFINED", UndefinedFilter );
-	IMAGICK_REGISTER_CONST_LONG("FILTER_POINT", PointFilter );
-	IMAGICK_REGISTER_CONST_LONG("FILTER_BOX", BoxFilter );
-	IMAGICK_REGISTER_CONST_LONG("FILTER_TRIANGLE", TriangleFilter );
-	IMAGICK_REGISTER_CONST_LONG("FILTER_HERMITE", HermiteFilter );
-	IMAGICK_REGISTER_CONST_LONG("FILTER_HANNING", HanningFilter );
-	IMAGICK_REGISTER_CONST_LONG("FILTER_HAMMING", HammingFilter );
-	IMAGICK_REGISTER_CONST_LONG("FILTER_BLACKMAN", BlackmanFilter );
-	IMAGICK_REGISTER_CONST_LONG("FILTER_GAUSSIAN", GaussianFilter );
-	IMAGICK_REGISTER_CONST_LONG("FILTER_QUADRATIC", QuadraticFilter );
-	IMAGICK_REGISTER_CONST_LONG("FILTER_CUBIC", CubicFilter );
-	IMAGICK_REGISTER_CONST_LONG("FILTER_CATROM", CatromFilter );
-	IMAGICK_REGISTER_CONST_LONG("FILTER_MITCHELL", MitchellFilter );
-	IMAGICK_REGISTER_CONST_LONG("FILTER_LANCZOS", LanczosFilter );
-	IMAGICK_REGISTER_CONST_LONG("FILTER_BESSEL", BesselFilter );
-	IMAGICK_REGISTER_CONST_LONG("FILTER_SINC", SincFilter );
-	IMAGICK_REGISTER_CONST_LONG("IMGTYPE_UNDEFINED", UndefinedType );
-	IMAGICK_REGISTER_CONST_LONG("IMGTYPE_BILEVEL", BilevelType );
-	IMAGICK_REGISTER_CONST_LONG("IMGTYPE_GRAYSCALE", GrayscaleType );
-	IMAGICK_REGISTER_CONST_LONG("IMGTYPE_GRAYSCALEMATTE", GrayscaleMatteType );
-	IMAGICK_REGISTER_CONST_LONG("IMGTYPE_PALETTE",  PaletteType );
-	IMAGICK_REGISTER_CONST_LONG("IMGTYPE_PALETTEMATTE", PaletteMatteType );
-	IMAGICK_REGISTER_CONST_LONG("IMGTYPE_TRUECOLOR", TrueColorType );
-	IMAGICK_REGISTER_CONST_LONG("IMGTYPE_TRUECOLORMATTE", TrueColorMatteType );
-	IMAGICK_REGISTER_CONST_LONG("IMGTYPE_COLORSEPARATION", ColorSeparationType );
-	IMAGICK_REGISTER_CONST_LONG("IMGTYPE_COLORSEPARATIONMATTE", ColorSeparationMatteType );
-	IMAGICK_REGISTER_CONST_LONG("IMGTYPE_OPTIMIZE", OptimizeType );
-	IMAGICK_REGISTER_CONST_LONG("RESOLUTION_UNDEFINED", UndefinedResolution );
-	IMAGICK_REGISTER_CONST_LONG("RESOLUTION_PIXELSPERINCH", PixelsPerInchResolution );
+	IMAGICK_REGISTER_CONST_LONG("STYLE_NORMAL", NormalStyle);
+	IMAGICK_REGISTER_CONST_LONG("STYLE_ITALIC", ItalicStyle);
+	IMAGICK_REGISTER_CONST_LONG("STYLE_OBLIQUE", ObliqueStyle);
+	IMAGICK_REGISTER_CONST_LONG("STYLE_ANY", AnyStyle);
+	IMAGICK_REGISTER_CONST_LONG("FILTER_UNDEFINED", UndefinedFilter);
+	IMAGICK_REGISTER_CONST_LONG("FILTER_POINT", PointFilter);
+	IMAGICK_REGISTER_CONST_LONG("FILTER_BOX", BoxFilter);
+	IMAGICK_REGISTER_CONST_LONG("FILTER_TRIANGLE", TriangleFilter);
+	IMAGICK_REGISTER_CONST_LONG("FILTER_HERMITE", HermiteFilter);
+	IMAGICK_REGISTER_CONST_LONG("FILTER_HANNING", HanningFilter);
+	IMAGICK_REGISTER_CONST_LONG("FILTER_HAMMING", HammingFilter);
+	IMAGICK_REGISTER_CONST_LONG("FILTER_BLACKMAN", BlackmanFilter);
+	IMAGICK_REGISTER_CONST_LONG("FILTER_GAUSSIAN", GaussianFilter);
+	IMAGICK_REGISTER_CONST_LONG("FILTER_QUADRATIC", QuadraticFilter);
+	IMAGICK_REGISTER_CONST_LONG("FILTER_CUBIC", CubicFilter);
+	IMAGICK_REGISTER_CONST_LONG("FILTER_CATROM", CatromFilter);
+	IMAGICK_REGISTER_CONST_LONG("FILTER_MITCHELL", MitchellFilter);
+	IMAGICK_REGISTER_CONST_LONG("FILTER_LANCZOS", LanczosFilter);
+	IMAGICK_REGISTER_CONST_LONG("FILTER_BESSEL", BesselFilter);
+	IMAGICK_REGISTER_CONST_LONG("FILTER_SINC", SincFilter);
+	IMAGICK_REGISTER_CONST_LONG("IMGTYPE_UNDEFINED", UndefinedType);
+	IMAGICK_REGISTER_CONST_LONG("IMGTYPE_BILEVEL", BilevelType);
+	IMAGICK_REGISTER_CONST_LONG("IMGTYPE_GRAYSCALE", GrayscaleType);
+	IMAGICK_REGISTER_CONST_LONG("IMGTYPE_GRAYSCALEMATTE", GrayscaleMatteType);
+	IMAGICK_REGISTER_CONST_LONG("IMGTYPE_PALETTE",  PaletteType);
+	IMAGICK_REGISTER_CONST_LONG("IMGTYPE_PALETTEMATTE", PaletteMatteType);
+	IMAGICK_REGISTER_CONST_LONG("IMGTYPE_TRUECOLOR", TrueColorType);
+	IMAGICK_REGISTER_CONST_LONG("IMGTYPE_TRUECOLORMATTE", TrueColorMatteType);
+	IMAGICK_REGISTER_CONST_LONG("IMGTYPE_COLORSEPARATION", ColorSeparationType);
+	IMAGICK_REGISTER_CONST_LONG("IMGTYPE_COLORSEPARATIONMATTE", ColorSeparationMatteType);
+	IMAGICK_REGISTER_CONST_LONG("IMGTYPE_OPTIMIZE", OptimizeType);
+	IMAGICK_REGISTER_CONST_LONG("RESOLUTION_UNDEFINED", UndefinedResolution);
+	IMAGICK_REGISTER_CONST_LONG("RESOLUTION_PIXELSPERINCH", PixelsPerInchResolution);
 	IMAGICK_REGISTER_CONST_LONG("RESOLUTION_PIXELSPERCENTIMETER", PixelsPerCentimeterResolution);
 	IMAGICK_REGISTER_CONST_LONG("COMPRESSION_UNDEFINED", UndefinedCompression);
 	IMAGICK_REGISTER_CONST_LONG("COMPRESSION_NO", NoCompression);
@@ -706,7 +847,7 @@ void initialize_imagick_constants()
 	IMAGICK_REGISTER_CONST_LONG("NOISE_GAUSSIAN", GaussianNoise);
 	IMAGICK_REGISTER_CONST_LONG("NOISE_MULTIPLICATIVEGAUSSIAN", MultiplicativeGaussianNoise);
 	IMAGICK_REGISTER_CONST_LONG("NOISE_IMPULSE", ImpulseNoise);
-	IMAGICK_REGISTER_CONST_LONG("NOISE_LAPLACIAN", LaplacianNoise );
+	IMAGICK_REGISTER_CONST_LONG("NOISE_LAPLACIAN", LaplacianNoise);
 	IMAGICK_REGISTER_CONST_LONG("NOISE_POISSON", PoissonNoise);
 #if MagickLibVersion > 0x635
 	IMAGICK_REGISTER_CONST_LONG("NOISE_RANDOM", RandomNoise);
@@ -754,6 +895,22 @@ void initialize_imagick_constants()
 	IMAGICK_REGISTER_CONST_LONG("EVALUATE_SET", SetEvaluateOperator);
 	IMAGICK_REGISTER_CONST_LONG("EVALUATE_SUBTRACT", SubtractEvaluateOperator);
 	IMAGICK_REGISTER_CONST_LONG("EVALUATE_XOR", XorEvaluateOperator);
+#if MagickLibVersion > 0x643
+	IMAGICK_REGISTER_CONST_LONG("EVALUATE_POW", PowEvaluateOperator);
+	IMAGICK_REGISTER_CONST_LONG("EVALUATE_LOG", LogEvaluateOperator);
+	IMAGICK_REGISTER_CONST_LONG("EVALUATE_THRESHOLD", ThresholdEvaluateOperator);
+	IMAGICK_REGISTER_CONST_LONG("EVALUATE_THRESHOLDBLACK", ThresholdBlackEvaluateOperator);
+	IMAGICK_REGISTER_CONST_LONG("EVALUATE_THRESHOLDWHITE", ThresholdWhiteEvaluateOperator);
+	IMAGICK_REGISTER_CONST_LONG("EVALUATE_GAUSSIANNOISE", GaussianNoiseEvaluateOperator);
+	IMAGICK_REGISTER_CONST_LONG("EVALUATE_IMPULSENOISE", ImpulseNoiseEvaluateOperator);
+	IMAGICK_REGISTER_CONST_LONG("EVALUATE_LAPLACIANNOISE", LaplacianNoiseEvaluateOperator);
+	IMAGICK_REGISTER_CONST_LONG("EVALUATE_MULTIPLICATIVENOISE", MultiplicativeNoiseEvaluateOperator);
+	IMAGICK_REGISTER_CONST_LONG("EVALUATE_POISSONNOISE", PoissonNoiseEvaluateOperator);
+	IMAGICK_REGISTER_CONST_LONG("EVALUATE_UNIFORMNOISE", UniformNoiseEvaluateOperator);
+	IMAGICK_REGISTER_CONST_LONG("EVALUATE_COSINE", CosineEvaluateOperator);
+	IMAGICK_REGISTER_CONST_LONG("EVALUATE_SINE", SineEvaluateOperator);
+	IMAGICK_REGISTER_CONST_LONG("EVALUATE_ADDMODULUS", AddModulusEvaluateOperator);
+#endif	
 	IMAGICK_REGISTER_CONST_LONG("COLORSPACE_UNDEFINED", UndefinedColorspace);
 	IMAGICK_REGISTER_CONST_LONG("COLORSPACE_RGB", RGBColorspace);
 	IMAGICK_REGISTER_CONST_LONG("COLORSPACE_GRAY", GRAYColorspace);
@@ -878,7 +1035,7 @@ void initialize_imagick_constants()
 #if MagickLibVersion > 0x634
 	IMAGICK_REGISTER_CONST_LONG("INTERPOLATE_SPLINE", SplineInterpolatePixel);
 #endif
-#if MagickLibVersion > 0x632
+#if MagickLibVersion > 0x628
 	IMAGICK_REGISTER_CONST_LONG("LAYERMETHOD_UNDEFINED", UndefinedLayer);
 	IMAGICK_REGISTER_CONST_LONG("LAYERMETHOD_COALESCE", CoalesceLayer);
 	IMAGICK_REGISTER_CONST_LONG("LAYERMETHOD_COMPAREANY", CompareAnyLayer);
@@ -888,7 +1045,7 @@ void initialize_imagick_constants()
 	IMAGICK_REGISTER_CONST_LONG("LAYERMETHOD_OPTIMIZE", OptimizeLayer);
 	IMAGICK_REGISTER_CONST_LONG("LAYERMETHOD_OPTIMIZEPLUS", OptimizePlusLayer);
 #endif
-#if MagickLibVersion > 0x633
+#if MagickLibVersion > 0x629
 	IMAGICK_REGISTER_CONST_LONG("LAYERMETHOD_OPTIMIZEIMAGE", OptimizeImageLayer);
 	IMAGICK_REGISTER_CONST_LONG("LAYERMETHOD_OPTIMIZETRANS", OptimizeTransLayer);
 	IMAGICK_REGISTER_CONST_LONG("LAYERMETHOD_REMOVEDUPS", RemoveDupsLayer);
@@ -923,7 +1080,7 @@ void initialize_imagick_constants()
 	IMAGICK_REGISTER_CONST_LONG("DISTORTION_SHEPARDS", ShepardsDistortion);
 	IMAGICK_REGISTER_CONST_LONG("DISTORTION_SENTINEL", SentinelDistortion);
 #endif
-#ifdef HAVE_IMAGEMAGICK6364ORLATER
+#if MagickLibVersion > 0x636
 	IMAGICK_REGISTER_CONST_LONG("LAYERMETHOD_MERGE", MergeLayer);
 	IMAGICK_REGISTER_CONST_LONG("LAYERMETHOD_FLATTEN", FlattenLayer);
 	IMAGICK_REGISTER_CONST_LONG("LAYERMETHOD_MOSAIC", MosaicLayer);
@@ -941,6 +1098,21 @@ void initialize_imagick_constants()
 	IMAGICK_REGISTER_CONST_LONG("ALPHACHANNEL_EXTRACT", ExtractAlphaChannel);
 	IMAGICK_REGISTER_CONST_LONG("ALPHACHANNEL_OPAQUE", OpaqueAlphaChannel);
 	IMAGICK_REGISTER_CONST_LONG("ALPHACHANNEL_SHAPE", ShapeAlphaChannel);
-	IMAGICK_REGISTER_CONST_LONG("ALPHACHANNEL_TRANSPARENT", TransparentAlphaChannel);	
+	IMAGICK_REGISTER_CONST_LONG("ALPHACHANNEL_TRANSPARENT", TransparentAlphaChannel);
+	IMAGICK_REGISTER_CONST_LONG("SPARSECOLORMETHOD_UNDEFINED", UndefinedColorInterpolate);
+	IMAGICK_REGISTER_CONST_LONG("SPARSECOLORMETHOD_BARYCENTRIC", BarycentricColorInterpolate);
+	IMAGICK_REGISTER_CONST_LONG("SPARSECOLORMETHOD_BILINEAR", BilinearColorInterpolate);
+	IMAGICK_REGISTER_CONST_LONG("SPARSECOLORMETHOD_POLYNOMIAL", PolynomialColorInterpolate);
+	IMAGICK_REGISTER_CONST_LONG("SPARSECOLORMETHOD_SPEPARDS", ShepardsColorInterpolate);
+	IMAGICK_REGISTER_CONST_LONG("SPARSECOLORMETHOD_VORONOI", VoronoiColorInterpolate);
+	IMAGICK_REGISTER_CONST_LONG("DITHERMETHOD_UNDEFINED", UndefinedDitherMethod);
+	IMAGICK_REGISTER_CONST_LONG("DITHERMETHOD_NO", NoDitherMethod);
+	IMAGICK_REGISTER_CONST_LONG("DITHERMETHOD_RIEMERSMA", RiemersmaDitherMethod);
+	IMAGICK_REGISTER_CONST_LONG("DITHERMETHOD_FLOYDSTEINBERG", FloydSteinbergDitherMethod);
+#endif
+#if MagickLibVersion > 0x648
+	IMAGICK_REGISTER_CONST_LONG("FUNCTION_UNDEFINED", UndefinedFunction);
+	IMAGICK_REGISTER_CONST_LONG("FUNCTION_POLYNOMIAL", PolynomialFunction);
+	IMAGICK_REGISTER_CONST_LONG("FUNCTION_SINUSOID", SinusoidFunction);
 #endif
 }
