@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP Version 5 / Imagick	                                          |
    +----------------------------------------------------------------------+
-   | Copyright (c) 2006-2008 Mikko Koppanen, Scott MacVicar               |
+   | Copyright (c) 2006-2009 Mikko Koppanen, Scott MacVicar               |
    | Imagemagick (c) ImageMagick Studio LLC                               |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
@@ -99,17 +99,24 @@ void count_pixeliterator_rows(php_imagickpixeliterator_object *internpix TSRMLS_
 #endif
 
 int count_occurences_of(char needle, char *haystack TSRMLS_DC );
-int read_image_into_magickwand(php_imagick_object *intern, char *filename, int type TSRMLS_DC );
-int write_image_from_filename(php_imagick_object *intern, char *filename, zend_bool adjoin, int type TSRMLS_DC );
-int check_configured_font(char *font, int font_len TSRMLS_DC );
-int check_write_access(char *absolute TSRMLS_DC );
+int read_image_into_magickwand(php_imagick_object *intern, int type, char *filename, int filename_len TSRMLS_DC);
+int write_image_from_filename(php_imagick_object *intern, char *filename, zend_bool adjoin, int type TSRMLS_DC);
+int check_configured_font(char *font, int font_len TSRMLS_DC);
+int check_write_access(char *absolute TSRMLS_DC);
+
+MagickBooleanType php_imagick_read_image_using_imagemagick(php_imagick_object *intern, int type, char *filename, int filename_len TSRMLS_DC);
+
+
 
 zend_bool crop_thumbnail_image(MagickWand *magick_wand, long desired_width, long desired_height TSRMLS_DC);
 char *get_pseudo_filename(char* pseudo_string TSRMLS_DC);
-double *get_double_array_from_zval(zval *param_array, long *num_elements TSRMLS_DC );
+double *get_double_array_from_zval(zval *param_array, long *num_elements TSRMLS_DC);
+long *get_long_array_from_zval(zval *param_array, long *num_elements TSRMLS_DC);
+unsigned char *get_char_array_from_zval(zval *param_array, long *num_elements TSRMLS_DC);
 
 MagickBooleanType php_imagick_progress_monitor(const char *text, const MagickOffsetType offset, const MagickSizeType span, void *client_data);
-
+int php_imagick_stream_handler(php_imagick_object *intern, php_stream *stream, char *filename, int type TSRMLS_DC);
+zend_bool php_imagick_validate_map(const char *map TSRMLS_DC);
 
 /* Define some color constants */
 #define IMAGICKCOLORBLACK 11
@@ -142,6 +149,12 @@ MagickBooleanType php_imagick_progress_monitor(const char *text, const MagickOff
 #define IMAGICK_READ_WRITE_FILENAME_TOO_LONG 5
 #define IMAGICK_READ_WRITE_PATH_DOES_NOT_EXIST 6
 
+/* Filehandle handling */
+#define IMAGICK_WRITE_IMAGE_FILE 1
+#define IMAGICK_WRITE_IMAGES_FILE 2
+#define IMAGICK_READ_IMAGE_FILE 3
+#define IMAGICK_PING_IMAGE_FILE 4
+
 /* Class entries */
 extern zend_class_entry *php_imagick_sc_entry;
 extern zend_class_entry *php_imagick_exception_class_entry;
@@ -203,15 +216,20 @@ PHP_METHOD(imagick, setimageorientation);
 #if MagickLibVersion > 0x634
 PHP_METHOD(imagick, paintfloodfillimage);
 #endif
-#ifdef HAVE_IMAGEMAGICK6359ORLATER
+#if MagickLibVersion > 0x635
 PHP_METHOD(imagick, clutimage);
 PHP_METHOD(imagick, getimageproperties);
 PHP_METHOD(imagick, getimageprofiles);
-#endif
-#if MagickLibVersion > 0x635
 PHP_METHOD(imagick, distortimage);
+PHP_METHOD(imagick, writeimagefile);
+PHP_METHOD(imagick, writeimagesfile);
+PHP_METHOD(imagick, resetimagepage);
+PHP_METHOD(imagick, setimageclipmask);
+PHP_METHOD(imagick, getimageclipmask);
+PHP_METHOD(imagick, animateimages);
+PHP_METHOD(imagick, recolorimage);
 #endif
-#if defined(HAVE_IMAGEMAGICK6364ORLATER)
+#if MagickLibVersion > 0x636
 PHP_METHOD(imagick, setfont);
 PHP_METHOD(imagick, getfont);
 PHP_METHOD(imagick, setpointsize);
@@ -219,15 +237,42 @@ PHP_METHOD(imagick, getpointsize);
 PHP_METHOD(imagick, mergeimagelayers);
 #endif
 #if MagickLibVersion > 0x637
+PHP_METHOD(imagick, floodfillpaintimage);
+PHP_METHOD(imagick, opaquepaintimage);
+PHP_METHOD(imagick, transparentpaintimage);
 PHP_METHOD(imagick, setimagealphachannel);
 #endif
 #if MagickLibVersion > 0x638
 PHP_METHOD(imagick, liquidrescaleimage);
+PHP_METHOD(imagick, decipherimage);
+PHP_METHOD(imagick, encipherimage);
 #endif
-#if MagickLibVersion >= 0x640
+#if MagickLibVersion > 0x639
 PHP_METHOD(imagick, setgravity);
 PHP_METHOD(imagick, getgravity);
 PHP_METHOD(imagick, getimagechannelrange);
+PHP_METHOD(imagick, getimagealphachannel);
+#endif
+#if MagickLibVersion > 0x642
+PHP_METHOD(imagick, getimagechanneldistortions);
+#endif
+#if MagickLibVersion > 0x643
+PHP_METHOD(imagick, getimagegravity);
+PHP_METHOD(imagick, setimagegravity);
+#endif
+#if MagickLibVersion > 0x645
+PHP_METHOD(imagick, importimagepixels);
+PHP_METHOD(imagick, deskewimage);
+PHP_METHOD(imagick, segmentimage);
+PHP_METHOD(imagick, sparsecolorimage);
+PHP_METHOD(imagick, remapimage);
+#endif
+#if MagickLibVersion > 0x646
+PHP_METHOD(imagick, exportimagepixels);
+#endif
+#if MagickLibVersion > 0x648
+PHP_METHOD(imagick, getimagechannelkurtosis);
+PHP_METHOD(imagick, functionimage);
 #endif
 PHP_METHOD(imagick, __construct);
 PHP_METHOD(imagick, __tostring);
@@ -259,6 +304,7 @@ PHP_METHOD(imagick, setimageformat);
 PHP_METHOD(imagick, setimagefilename);
 PHP_METHOD(imagick, getimagefilename);
 PHP_METHOD(imagick, getimageformat);
+PHP_METHOD(imagick, getimagemimetype);
 PHP_METHOD(imagick, getimagesize);
 PHP_METHOD(imagick, getimagegeometry);
 PHP_METHOD(imagick, getimagecolors);
@@ -479,6 +525,12 @@ PHP_METHOD(imagick, settype);
 #if MagickLibVersion > 0x628
 PHP_METHOD(imagickdraw, resetvectorgraphics);
 #endif
+#if MagickLibVersion > 0x649
+PHP_METHOD(imagickdraw, gettextkerning);
+PHP_METHOD(imagickdraw, settextkerning);
+PHP_METHOD(imagickdraw, gettextinterwordspacing);
+PHP_METHOD(imagickdraw, settextinterwordspacing);
+#endif
 PHP_METHOD(imagickdraw, __construct);
 PHP_METHOD(imagickdraw, setfillcolor);
 PHP_METHOD(imagickdraw, setfillalpha);
@@ -512,6 +564,7 @@ PHP_METHOD(imagickdraw, polygon);
 PHP_METHOD(imagickdraw, arc);
 PHP_METHOD(imagickdraw, matte);
 PHP_METHOD(imagickdraw, point);
+PHP_METHOD(imagickdraw, getfontstretch);
 PHP_METHOD(imagickdraw, setfontstretch);
 PHP_METHOD(imagickdraw, settextalignment);
 PHP_METHOD(imagickdraw, settextdecoration);
@@ -616,6 +669,10 @@ PHP_METHOD(imagickpixeliterator, valid);
 #if MagickLibVersion > 0x628
 PHP_METHOD(imagickpixel, gethsl);
 PHP_METHOD(imagickpixel, sethsl);
+PHP_METHOD(imagickpixel, getcolorvaluequantum);
+PHP_METHOD(imagickpixel, setcolorvaluequantum);
+PHP_METHOD(imagickpixel, getindex);
+PHP_METHOD(imagickpixel, setindex);
 #endif
 PHP_METHOD(imagickpixel, __construct);
 PHP_METHOD(imagickpixel, setcolor);
