@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP Version 5 / Imagick	                                          |
    +----------------------------------------------------------------------+
-   | Copyright (c) 2006-2008 Mikko Koppanen, Scott MacVicar               |
+   | Copyright (c) 2006-2009 Mikko Koppanen, Scott MacVicar               |
    | Imagemagick (c) ImageMagick Studio LLC                               |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
@@ -154,16 +154,19 @@
 		} \
 	} \
 
-#define IMAGICK_HAS_FORMAT(buffer, magick_wand)\
+#define IMAGICK_HAS_FORMAT(buffer, magick_wand, free_buffer)\
 	buffer = MagickGetImageFormat(magick_wand);\
 	if(buffer == (char *)NULL || *buffer == '\0') {\
 		IMAGICK_FREE_MEMORY(char *, buffer); IMAGICK_THROW_EXCEPTION_WITH_MESSAGE(IMAGICK_CLASS, "Image has no format", 1); \
 	} else { \
-		IMAGICK_FREE_MEMORY(char *, buffer); \
+		if (free_buffer) IMAGICK_FREE_MEMORY(char *, buffer); \
 	} \
 
-#define IMAGICK_METHOD_DEPRECATED(className, methodName)\
-	php_error(E_STRICT, "%s::%s method is deprecated and it's use should be avoided", className, methodName);
+#define IMAGICK_METHOD_DEPRECATED(class_name, method_name) \
+	php_error(E_STRICT, "%s::%s method is deprecated and it's use should be avoided", class_name, method_name);
+
+#define IMAGICK_METHOD_DEPRECATED_USE_INSTEAD(class_name, method_name, new_class, new_method) \
+	php_error(E_STRICT, "%s::%s is deprecated. %s::%s should be used instead", class_name, method_name, new_class, new_method);
 
 #define IMAGICK_CHECK_READ_OR_WRITE_ERROR(intern, filename, error, free, message) \
 	switch (error) { \
@@ -258,10 +261,12 @@
 	MagickSetLastIterator(intern->magick_wand);
 
 #define IMAGICK_REPLACE_MAGICKWAND(intern, new_wand)\
-	if (intern->magick_wand != (MagickWand *)NULL) {\
-		intern->magick_wand = (MagickWand *)DestroyMagickWand(intern->magick_wand);\
-		intern->magick_wand = new_wand;\
-	} \
+	if (intern->magick_wand == (MagickWand *)NULL) {\
+		intern->magick_wand = new_wand; \
+	} else { \
+		intern->magick_wand = (MagickWand *)DestroyMagickWand(intern->magick_wand); \
+		intern->magick_wand = new_wand; \
+	}
 
 #define IMAGICKPIXEL_REPLACE_PIXELWAND(intern, new_wand)\
 	if(intern->pixel_wand != (PixelWand *)NULL && intern->initialized_via_iterator != 1) {\
@@ -271,10 +276,12 @@
 		intern->pixel_wand = new_wand;\
 	} \
 
-#define IMAGICKDRAW_REPLACE_DRAWINGWAND(intern, new_wand)\
-	if (intern->drawing_wand != (DrawingWand *)NULL) {\
-		intern->drawing_wand = (DrawingWand *)DestroyDrawingWand(intern->drawing_wand);\
-		intern->drawing_wand = new_wand;\
+#define IMAGICKDRAW_REPLACE_DRAWINGWAND(intern, new_wand) \
+	if (intern->drawing_wand == (DrawingWand *)NULL) { \
+		intern->drawing_wand = new_wand; \
+	} else { \
+		intern->drawing_wand = (DrawingWand *)DestroyDrawingWand(intern->drawing_wand); \
+		intern->drawing_wand = new_wand; \
 	} \
 
 #define IMAGICK_CAST_PARAMETER_TO_COLOR(param, internp, caller) \
@@ -284,7 +291,8 @@
 			zval *object; \
 			PixelWand *pixel_wand = NewPixelWand(); \
 			if (!PixelSetColor(pixel_wand, Z_STRVAL_P(param))) { \
-				IMAGICK_THROW_IMAGICKPIXEL_EXCEPTION(pixel_wand, "Unrecognized color string", 3); \
+				pixel_wand = DestroyPixelWand(pixel_wand); \
+				IMAGICK_THROW_EXCEPTION_WITH_MESSAGE(caller, "Unrecognized color string", caller); \
 				return; \
 			} \
 			MAKE_STD_ZVAL(object); \
@@ -296,7 +304,11 @@
 		} \
 		break; \
 		case IS_OBJECT: \
-			internp = (php_imagickpixel_object *)zend_object_store_get_object(param TSRMLS_CC); \
+			if (instanceof_function_ex(Z_OBJCE_P(param), php_imagickpixel_sc_entry, 0 TSRMLS_CC)) { \
+				internp = (php_imagickpixel_object *)zend_object_store_get_object(param TSRMLS_CC); \
+			} else { \
+				IMAGICK_THROW_EXCEPTION_WITH_MESSAGE(caller, "The parameter must be an instance of ImagickPixel or a string", (long)caller); \
+			} \
 		break; \
 		default: \
 			IMAGICK_THROW_EXCEPTION_WITH_MESSAGE(caller, "Invalid parameter provided", (long)caller); \
@@ -320,7 +332,11 @@
 		} \
 		break; \
 		case IS_OBJECT: \
-			internp = (php_imagickpixel_object *)zend_object_store_get_object(param TSRMLS_CC); \
+			if (instanceof_function_ex(Z_OBJCE_P(param), php_imagickpixel_sc_entry, 0 TSRMLS_CC)) { \
+				internp = (php_imagickpixel_object *)zend_object_store_get_object(param TSRMLS_CC); \
+			} else { \
+				IMAGICK_THROW_EXCEPTION_WITH_MESSAGE(caller, "The parameter must be an instance of ImagickPixel or a string", (long)caller); \
+			} \
 		break; \
 		default: \
 			IMAGICK_THROW_EXCEPTION_WITH_MESSAGE(caller, "Invalid parameter provided", (long)caller); \
@@ -339,25 +355,12 @@
 		if ((orig_width <= desired_width) && (orig_height <= desired_height)) { \
 			RETURN_TRUE; \
 		} \
-		if ((orig_width / desired_width) > (orig_height / desired_height)) { \
+		if ((orig_width - desired_width) > (orig_height - desired_height)) { \
 			ratio = (double)orig_width / (double)desired_width; \
 			desired_height = (double)orig_height / ratio; \
-		} else if ((orig_height / desired_height) > (orig_width / desired_width)) { \
+		} else { \
 			ratio = (double)orig_height / (double)desired_height; \
 			desired_width = (double)orig_width / ratio; \
-		} else { \
-			if (orig_height > orig_width) { \
-				desired_width = 0; \
-			} else { \
-				desired_height = 0; \
-			} \
-			if (desired_width <= 0) { \
-				ratio = (double)orig_height / (double)desired_height; \
-				desired_width = orig_width / ratio; \
-			} else if (desired_height <= 0) { \
-				ratio = (double)orig_width / (double)desired_width; \
-				desired_height = orig_height / ratio; \
-			} \
 		} \
 		if (desired_width < 1) { \
 			desired_width = 1; \
